# SQLITE 3.41

## Prérequis

- Dernière version sqlite 3.41

## Resources

- https://www.sqlite.org/
- https://www.sqlitetutorial.net
- https://www.tutorialspoint.com/sqlite


## Démarrer sqlite

```bash
~$ sqlite3
SQLite version 3.41.2 2023-03-17 14:22:45
Enter ".help" for usage hints.
Connected to a transient in-memory database.
Use ".open FILENAME" to reopen on a persistent database.
sqlite> 
```

Puis, configurer **à chaque fois** les instructions suivantes pour avoir des contraintes strictes et pour forcer l'emploi des clés étrangères.

```bash
sqlite> PRAGMA foreign_keys = ON;
```

Enfin, pour un peu de cosmétique afin de mieux lire le résultat des requêtes.

```bash
sqlite> .mode column
sqlite> .headers on
```

### Ouvrir ou créer une base de données

```bash
sqlite3 test.db
```

```bash
sqlite3
sqlite> .open test.db
```

### Exécuter un script sql

```bash
sqlite> .read monscript.sql
```

### Autres commandes

```bash
sqlite> .tables # liste les tables
sqlite> .fullschema # liste les schémas détaillés des tables
sqlite> .schema Conference # liste le schéma d'une table spécifique
sqlite> ATTACH conf.db as conference # Utiliser une base de données 
sqlite> DETACH conference # Fermer une base de données 
```


# Création d'une base de données

## Système de type

Le système de type de sqlite est très flexible. Il n'est pas nécessaire de préciser un type et celui qui est donné est conseillé, mais non obligatoire (https://www.sqlite.org/flextypegood.html). Il est dès lors possible de créer une table ainsi : `CREATE TABLE Car(immatriculation, color, owner);`.

Voici un extrait de la documentation officielle (https://www.sqlite.org/stricttables.html):

"_So an attempt to insert the string '123' results in an integer 123 being inserted. But if the content cannot be losslessly converted into an integer, for example if the input is 'xyz', then the original string is inserted instead_".

Ce choix est sujet à débats et à mon avis il offre plus d'inconvénients que d'avantages. 

Heureusement, depuis la version 3.37, sqlite offre le mode STRICT pour renforcer le système de type pour chaque table individuellement : `CREATE TABLE t (i INTEGER, t TEXT) STRICT;` (https://www.sqlite.org/stricttables.html).

Dans ce cas, le typage doit être spécifié, si une conversion implicite n'est pas possible, une erreur est levée. Le type `ANY` peut toutefois être employé pour indiquer qu'une colonne peut comporter différents types.


## Création d'une table (préférez les tables strictes)

```sql
CREATE TABLE Test (
    i INTEGER NOT NULL PRIMARY KEY,
    t TEXT NOT NULL,
    r REAL,
    a ANY
) STRICT;
```

## Création d'une table avec clé auto incrémentée

```sql
CREATE TABLE Test (
    i INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
    t TEXT,
) STRICT;
```

## Création d'une table avec clé composée de plusieurs attributs

```sql
CREATE TABLE Test (
    i INTEGER NOT NULL,
    j INTEGER NOT NULL,
    t TEXT,
    PRIMARY KEY (i,j)
) STRICT;
```

## Création d'une table avec contraintes

```sql
CREATE TABLE Test (
    i INTEGER NOT NULL PRIMARY KEY,
    j INTEGER NOT NULL,
    k INTEGER NOT NULL,
    CHECK (j > 0 && j < 100),
    UNIQUE (j,k),
    FOREIGN KEY (k) REFERENCES Table(k)
) STRICT;
```

## Suppression d'une table avec ses enregistrements

```sql
DROP TABLE Test;
```

## Trigger

- https://www.sqlite.org/lang_createtrigger.html
- https://www.sqlitetutorial.net/sqlite-trigger/

### Syntaxe générale

```sql
CREATE TRIGGER [IF NOT EXISTS] trigger_name 
   [BEFORE|AFTER|INSTEAD OF] [INSERT|UPDATE|DELETE] 
   ON table_name
   [WHEN condition]
BEGIN
 statements;
END;
```

Suppression

```sql
DROP TRIGGER trigger_name;
```

### Exemple 1 

L'insertion est possible si la valeur de l'attribut `j` du nouvel enregistrement n'existe pas dans la table `Table`.

```sql
CREATE TRIGGER trigger_name BEFORE INSERT ON Test
BEGIN
SELECT
  CASE WHEN NEW.j IN (SELECT j FROM Table) THEN RAISE(ABORT, 'valid')
  END;
END; 
```

`RAISE` est une fonction.

ce trigger réalise exactement la même chose

```sql
CREATE TRIGGER trigger_name BEFORE INSERT ON Test WHEN NEW.i == 10
BEGIN
SELECT
    Raise(ABORT, 'valid');
END; 
```

### Exemple 2 

CREATE TRIGGER trigger_name BEFORE UPDATE ON Test 
BEGIN
    INSERT INTO Log VALUES (NEW.i, OLD.i);
END; 


# View

Une vue est un moyen d'enregistrer une requête dans une base de données. Elle peut être ensuite accessible aux utilisatateurs de la base.

Elle permette de cacher la complexité d'une requête ou de réduire la visibilité de certains champs.


## Syntaxe

Crétation

```sql
CREATE [TEMP] VIEW [IF NOT EXISTS] view_name[(column-name-list)]
AS 
select-statement;
```

Suppression

```sql
DROP VIEW view_name;
```

## Exemple

```sql
CREATE VIEW IF NOT EXISTS conf_price
AS SELECT id_conference, price FROM Conference;
...
SELECT * FROM conf_price;
```

ou

```sql
CREATE VIEW IF NOT EXISTS conf_price(id, price, topic)
AS SELECT id_conference, price, title FROM Conference;
...
SELECT id, topic FROM conf_price;
```

# Index

Un index est une structure de données additionnelle qui permet d'améliorer les performances des requêtes. La structure utilisée est un B-Arbre. Il offre des performances sur les égalités (=) et les plages de valeurs (`<`, `<=`, `>`, `>=`).

## Syntaxe générale

```sql
CREATE [UNIQUE] INDEX index_name 
ON table_name(column_list);
```

- un index unique (optionnel) n'autorise qu'une valeur pour un attribut ou groupe d'attributs
- des indexes personnalisés peuvent être créer (voir documentation officielle, "Expression-based Index")


## Exemple 1


- Recherche efficace d'un email
- Qu'un email ne peut exister dans la table `Person`.

```sql
CREATE UNIQUE INDEX index_email
ON Person(email);
```

## Exemple 2

- recherche optimisée sur la ville

```sql
CREATE INDEX index_conf
ON Hotel(city);
```

## Exemple 3


- recherche optimisée sur des plages de dates
- recherche optimisée sur la date de début uniquement
- recherche optimisée sur la date de début ET (suivi de) la date de fin
- non optimisé sur la date de fin uniquement

```sql
CREATE INDEX index_conf
ON Conference(start_date, end_date);
```

## Divers

- listez les indexes d'une table : `PRAGMA index_list('table_name');`
- supprimer un index: `DROP INDEX [IF EXISTS] index_name;`

# Explain

Comprendre l'optimisateur de requêtes à l'aide de :

```sql
EXPLAIN QUERY PLAN <select expression>
```
